package com.biganto.visual.roompark.conductor

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.annotation.LayoutRes
import androidx.annotation.StringRes
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.ICollapsingToolBar
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.mosby.mvi.BigantoMviController
import com.biganto.visual.roompark.di.dagger.PerScreen
import com.biganto.visual.roompark.view_utils.snackbar.ISnackBarProvider
import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.disposables.CompositeDisposable
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 28.05.2018.
 */

@PerScreen
abstract class BigantoBaseController<VS : BigantoBaseViewState,V: BigantoBaseContract<VS>,P : MviBasePresenter<V,VS>>
    : BigantoMviController<V, P> {

    constructor():super()

    constructor(args: Bundle):super(args)

    protected val detachDisposable = CompositeDisposable()

    abstract override fun createPresenter(): P

    @LayoutRes
    protected abstract fun getLayoutId(): Int

    @StringRes
    protected abstract fun getToolbarTitleId(): Int


    @Inject
    private lateinit var roomParkActivity:RoomParkMainActivity

    protected val toolBar: ICollapsingToolBar = roomParkActivity
    protected val snackbar: ISnackBarProvider = roomParkActivity.snackbarProvider

    override fun onDetach(view: View) {
        detachDisposable.clear()
        super.onDetach(view)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)
//        instantiate the view
        ButterKnife.bind(this, view)

        return view
    }

    protected fun View.hideKeyboard() {
        val inputMethodManager = applicationContext
            ?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.hideSoftInputFromWindow(this.windowToken, 0)
    }

    protected fun View.showKeyboard() {
        val inputMethodManager = applicationContext?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.showSoftInput(this, 0)
    }

    protected fun showMessage(@StringRes msgResId: Int) {
        Toast.makeText(this.applicationContext, msgResId, Toast.LENGTH_SHORT).show()
    }

    override fun handleBack(): Boolean {
        return super.handleBack()
    }

    protected val isTablet
        get() = resources?.getBoolean(R.bool.isTablet)?:false

    protected val catalogSpansCount
        get() = resources?.getInteger(R.integer.catalogSpans)?:1

    protected val toursSpansCount
        get() = resources?.getInteger(R.integer.catalogSpans)?:1
}