package com.biganto.visual.roompark.di.dagger

import android.app.Activity
import android.content.Context
import android.content.res.Resources
import com.biganto.visual.roompark.base.BaseRoomParkActivity
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.view_utils.snackbar.ISnackBarProvider
import com.biganto.visual.roompark.view_utils.snackbar.SnackBarProvider
import dagger.Binds
import dagger.Module
import dagger.Provides
import dagger.android.ContributesAndroidInjector

/**
 * Created by Vladislav Bogdashkin on 31.07.2019.
 */

@Module
abstract class ActivityModule {

    @Binds
    abstract fun providesContext(activity: Activity):Context

    @Module
    companion object{
        @JvmStatic
        @Provides
        fun provideResources(activity: Activity) : Resources = activity.resources

        @JvmStatic
        @Provides
        fun provideSnackBar(activity: Activity): ISnackBarProvider{
            return SnackBarProvider(activity)
        }

    }

}

@Module
abstract class AppActivityModule  {

    @Module
    internal interface MainActivityModule {
        @Binds
        fun bindMainActivity(activity: RoomParkMainActivity): Activity
    }


    @Module
    internal interface BaseActivityModule {
        @Binds
        fun bindBaseActivity(activity: BaseRoomParkActivity): Activity
    }


    @ContributesAndroidInjector(
        modules = [ActivityModule::class, MainActivityModule::class])
    abstract fun contributeMainActivityInjector(): RoomParkMainActivity


    @ContributesAndroidInjector(
        modules = [ActivityModule::class, BaseActivityModule::class])
    abstract fun contributeBaseActivityInjector(): BaseRoomParkActivity

}
